/*++

Copyright (c) 1996 bsquare consulting, inc.

Module:

     proteus.h

Abstract:

     Defines for the AMD ELAN(tm) SC400 reference platform

Author(s):

     Jeff McLeman

Revision(s):

    X-1      Curtisj     17-OCT-1997

                     Added define INTR_NETWORK (5)to support NE2000 interrupt.

--*/

#ifndef _PROTEUS_H_
#define _PROTEUS_H_

#include <winioctl.h>


//
// Defines for Proteus Reference Platform control registers
//

enum TINTR
   {
      INTR_TIMER          =   0 , // Bios Timer Tick
      INTR_KEYBOARD       =   1 , // AT keyboard (Eval board)
      INTR_CHAIN          =   2 , // really 9 from second controller
      INTR_PCMCIA_DATA    =   3 , // Used for the data Interrupt on the PCMCIA Controller (DO NOT CHANGE)  See Socket.c
      INTR_IRDA           =   4 , // COM1 in Infrared mode
      INTR_COM1           =   4 , // Serial  COM1
      INTR_NETWORK        =   5 , // was previously Alps Panel (Eval Board)
      INTR_I2C            =   6 , // Phillips I2C Chip
      INTR_QUADUART       =   7 , // StarTech Quad Uart
      INTR_RTC            =   8 , // RTC
      INTR_TOUCH          =   9 , // Touch panel interrupt from TriTech
      INTR_AUDIO          =   11, // Sound
      INTR_PCMCIA         =   15, // Used for the Status Change Interrupt on the PCMCIA Controller (DO NOT CHANGE) See Socket.c
      INTR_0=0, INTR_1, INTR_2,  INTR_3,  INTR_4,  INTR_5,  INTR_6,  INTR_7,
      INTR_8,   INTR_9, INTR_10, INTR_11, INTR_12, INTR_13, INTR_14, INTR_15,
      INTR_ILLEGAL        =   16, // To mark an illegal interrupt value
      INTR_COM2           =   17, // Special value to help with QUAD Port interrupt mapping
      INTR_COM3           =   18, // Special value to help with QUAD Port interrupt mapping
      INTR_COM4           =   20, // Special value to help with QUAD Port interrupt mapping
      INTR_COM5           =   24  // Special value to help with QUAD Port interrupt mapping
   };

#define PHYS_ADDR_RAM_BASE      0x00000000

#define PHYS_ADDR_BOOT_ROM_BASE 0x00800000

#define IOBASE_CPU_INDEX (unsigned char *)0x22      // Internal registers index
#define IOBASE_CPU_DATA  (unsigned char *)0x23      // Internal Registers data

//
// The following are defines for the SC400 internal registers at port
// 22h/23h.
//

//
// DRAM Config
//

#define SC400_DRAM_BNK0_CFG               0x00
#define SC400_DRAM_BNK1_CFG               0x01
#define SC400_DRAM_BNK2_CFG               0x02
#define SC400_DRAM_BNK3_CFG               0x03
#define SC400_DRAM_CTL_REG                0x04
#define SC400_DRAM_REFRESH_CTL            0x05
#define SC400_DRAM_DRV_CTL_A              0x06
#define SC400_DRAM_DRV_CTL_B              0x07

//
// Non cachable memory registers
//

#define SC400_NCACHE_WIN0                 0x10
#define SC400_NCACHE_WIN0_ATTRIB          0x11
#define SC400_NCACHE_WIN1                 0x12
#define SC400_NCACHE_WIN1_ATTRIB          0x13
#define SC400_CACHE_VL_REG                0x14

//
// ROM config
//

#define SC400_PIN_REG                     0x20
#define SC400_ROM0_SHADOW                 0x21
#define SC400_ROM0_ATTRIB                 0x22
#define SC400_ROMCS0_CFG_A                0x23
#define SC400_ROMCS0_CFG_B                0x24
#define SC400_ROMCS1_CFG_A                0x25
#define SC400_ROMCS1_CFG_B                0x26
#define SC400_ROMCS2_CFG_A                0x27
#define SC400_ROMCS2_CFG_B                0x28

//
// memory mapping
//

#define SC400_MMS_PAGE_ATTRIB             0x30
#define SC400_MMS_PAGE_DEV_SEL            0x31
#define SC400_MMS_PAGE5_DEST              0x32
#define SC400_MMS_PAGE5_ATTRIB            0x33
#define SC400_MMS_PAGE6_DEST              0x34
#define SC400_MMS_PAGE6_ATTRIB            0x35

//
// Pin Mux Register
//

#define SC400_PIN_MUX_A                                   0x38
#define SC400_PIN_MUX_B                                   0x39
#define SC400_PIN_MUX_C                                   0x3A


//
// GPI Termination
//
#define SC400_GPIO_TERM_CTL_A             0x3b
#define SC400_GPIO_TERM_CTL_B             0x3c
#define SC400_GPIO_TERM_CTL_C             0x3d
#define SC400_GPIO_TERM_CTL_D             0x3e

//
// Power managment
//

#define SC400_PMU_FORCE_MODE              0x40
#define SC400_PMU_LAST_MODE               0x41
#define SC400_PMU_HYPER_TIMER             0x42
#define SC400_PMU_STANDBY_TIMER           0x43
#define SC400_PMU_SUSPEND_LOW             0x44
#define SC400_PMU_WAKE_HIGH               0x45

//
// Wake up control and status
//

#define SC400_SUS_RES_PIN                 0x50
#define SC400_WAKE_SOURCE_ENA_A           0x52
#define SC400_WAKE_SOURCE_ENA_B           0x53
#define SC400_WAKE_SOURCE_ENA_C           0x54
#define SC400_WAKE_SOURCE_ENA_D           0x55
#define SC400_WAKE_SOURCE_STS_A           0x56
#define SC400_WAKE_SOURCE_STS_B           0x57
#define SC400_WAKE_SOURCE_STS_C           0x58
#define SC400_WAKE_SOURCE_STS_D           0x59

//
// Activity source
//

#define SC400_ACTIVE_SRC_ENA_A            0x62
#define SC400_ACTIVE_SRC_ENA_B            0x63
#define SC400_ACTIVE_SRC_ENA_C            0x64
#define SC400_ACTIVE_SRC_ENA_D            0x65
#define SC400_ACTIVE_SRC_STS_A            0x66
#define SC400_ACTIVE_SRC_STS_B            0x67
#define SC400_ACTIVE_SRC_STS_C            0x68
#define SC400_ACTIVE_SRC_STS_D            0x69
#define SC400_ACTIVE_CLASS_A              0x6A
#define SC400_ACTIVE_CLASS_B              0x6B
#define SC400_ACTIVE_CLASS_C              0x6C
#define SC400_ACTIVE_CLASS_D              0x6D

//
// Battery info
//

#define SC400_BATT_AC_CFG_A               0x70
#define SC400_BATT_AC_CFG_B               0x71
#define SC400_BATT_AC_PIN_STATE           0x72

//
// CPU clock control
//

#define SC400_CPU_CLOCK_SPEED             0x80
#define SC400_CPU_CLOCK_AUTO              0x81
#define SC400_CPU_CLOCK_CTL               0x82
#define SC400_CLOCK_IO_SELECT             0x83

//
// SMI/NMI Registers
//

#define SC400_SMI_MISC_ENA                0x90
#define SC400_SMI_PC_CARD_KB_ENA          0x91
#define SC400_SMI_MODE_TIMER_ENA          0x92
#define SC400_SMI_BATT_AC_ENA             0x93
#define SC400_SMI_MISC_STAT_A             0x94
#define SC400_SMI_PC_CARD_KB_STAT         0x95
#define SC400_SMI_MODE_TIMER_STAT         0x96
#define SC400_SMI_BATT_AC_STAT            0x97
#define SC400_SMI_SELECT                  0x98
#define SC400_SMI_IOACC_ENA_A             0x99
#define SC400_SMI_IOACC_ENA_B             0x9a
#define SC400_SMI_IOACC_STAT_A            0x9b
#define SC400_SMI_IOACC_STAT_B            0x9c
#define SC400_XMI_CONTROL                 0x9d

//
// GPIO_CS functions
//

#define SC400_GPIO_FUNC_SEL_A             0xa0
#define SC400_GPIO_FUNC_SEL_B             0xa1
#define SC400_GPIO_FUNC_SEL_C             0xa2
#define SC400_GPIO_FUNC_SEL_D             0xa3
#define SC400_GPIO_FUNC_SEL_E             0xa4
#define SC400_GPIO_FUNC_SEL_F             0xa5

#define SC400_GPIO_RW_REG_A               0xa6
#define SC400_GPIO_RW_REG_B               0xa7
#define SC400_GPIO_RW_REG_C               0xa8
#define SC400_GPIO_RW_REG_D               0xa9

#define SC400_GPIO_PMUA_MODE              0xaa
#define SC400_GPIO_PMUB_MODE              0xab
#define SC400_GPIO_PMUC_MODE              0xac
#define SC400_GPIO_PMUD_MODE              0xad

#define SC400_GPIO_PMU_MAP_A              0xae
#define SC400_GPIO_PMU_MAP_B              0xaf

#define SC400_XMI_GPIO_CS_MAP             0xb0
#define SC400_DECODE_GPIO_CS_MAP          0xb1
#define SC400_CS_GPIO_MAP_A               0xb2
#define SC400_CS_GPIO_MAP_B               0xb3
#define SC400_CSA_ADDR_DECODE             0xb4
#define SC400_GP_CSA_ADDR_MASK            0xb5

#define SC400_GP_CS_IO_QUAL               0xb8

//
// Keyboard
//

#define SC400_KB_CONFIG_A                 0xc0
#define SC400_KB_CONFIG_B                 0xc1
#define SC400_KB_IBUF_RD_BK               0xc2
#define SC400_KB_OBUF_WRITE               0xc3
#define SC400_KB_MOUSE_OBUF               0xc4
#define SC400_KB_STAT_WRITE               0xc5
#define SC400_KB_TIMER                    0xc6
#define SC400_KB_COLUMN                   0xc7
#define SC400_KB_ROW_A                    0xc8
#define SC400_KB_ROW_B                    0xc9
#define SC400_KB_COLUMN_TERM              0xca

//
// Extended feature and perphial control/enables
//

#define SC400_AT_IO_DISABLE_CFG           0xd0
#define SC400_AT_SER_PAR_CFG              0xd1
#define SC400_AT_PAR_CFG                  0xd2
#define SC400_AT_UART_FIFO_CTL            0xd3
#define SC400_AT_INT_CFG_A                0xd4
#define SC400_AT_INT_CFG_B                0xd5
#define SC400_AT_INT_CFG_C                0xd6
#define SC400_AT_INT_CFG_D                0xd7
#define SC400_AT_INT_CFG_E                0xd8
#define SC400_AT_DMA_LOW_PAGE             0xd9
#define SC400_AT_DMA_HIGH_PAGE            0xda
#define SC400_AT_RES_CHAN_MAP_A           0xdb
#define SC400_AT_RES_CHAN_MAP_B           0xdc
#define SC400_AT_GRAPH_CTL_A              0xdd
#define SC400_AT_GRAPH_CTL_B              0xde

//
// Suspend State Registers
//
#define SC400_SUSPEND_PIN_STATE_REG_A     0xe3
#define SC400_SUSPEND_PIN_STATE_REG_B     0xe4
//
// IRDA registers
//

#define SC400_IRDA_CTL                    0xeA
#define SC400_IRDA_STATUS                 0xeb
#define SC400_IRDA_CRC_STAT               0xec
#define SC400_IRDA_OWN_ADDR               0xed
#define SC400_IRDA_FR_LEN_A               0xee
#define SC400_IRDA_FR_LEN_B               0xef

//
// PCMCIA features
//

#define SC400_PCMCIA_EXT_FEATURE          0xf0
#define SC400_PCMCIA_MODE_DMA             0xf1
#define SC400_PCMCIA_SKT_TERM             0xf2

//
// System Management bus
//

#define SC400_SMB_OWN_ADDR                0xf8
#define SC400_SMB_XMIT_HOLD               0xf9
#define SC400_SMB_REC_HOLD                0xfa
#define SC400_SMB_CTL                     0xfb
#define SC400_SMB_STATUS                  0xfc


//
// Misc
//

#define SC400_MISC                        0xff
#define SC400_ELAN_REVISION               0xff

//
// end of SC400 IPR defs
//



//
// Serial debug port defs
//

#define COM1_BASE 0x3f8
#define COM2_BASE 0x2f8

#define comTxBuffer         0x00
#define comRxBuffer         0x00
#define comDivisorLow       0x00
#define comDivisorHigh      0x01
#define comIntEnable        0x01
#define comIntId            0x02
#define comFIFOControl      0x02
#define comLineControl      0x03
#define comModemControl     0x04
#define comLineStatus       0x05
#define comModemStatus      0x06


//
// internal CSC register access macros
//

#define CPU_INDEX 0x22
#define CPU_DATA  0x23

#ifdef _WIN32_WCE

#ifdef __cplusplus
extern "C" {
#endif

int __cdecl _inp(unsigned short);
unsigned short __cdecl _inpw(unsigned short);
unsigned long __cdecl _inpd(unsigned short);

int __cdecl _outp(unsigned short, int);
unsigned short __cdecl _outpw(unsigned short, unsigned short);
unsigned long __cdecl _outpd(unsigned short, unsigned long);

#ifdef __cplusplus
}
#endif

#pragma intrinsic(_inp, _inpw, _inpd, _outp, _outpw, _outpd)
#endif // _WIN32_WCE


//////////////////////// SC400 REGISTER FUNCTIONS ////////////////////////////////////////


_inline void WRITE_CSC_REGISTERS(UINT8 Index, UINT8 Data)
// JDW Save's flags now, uses word write
{
#ifdef _WIN32_WCE
   _asm
   {
       mov al, Index
       mov ah, Data
       out CPU_INDEX, ax
    }
#endif // _WIN32_WCE
}


_inline UCHAR READ_CSC_REGISTERS(UINT8 Index)
// JDW Save's flags now
{
#ifdef _WIN32_WCE
   char tmp;
   _asm
   {
      mov al, Index
      pushfd
      cli
      out CPU_INDEX, al
      in al,CPU_DATA
      popfd
      mov tmp, al
   }
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
}

_inline UCHAR OR_CSC_REGISTERS(UINT8 Index, UINT8 Data)
// JDW - New function to prevent any possibility of conflicts
{
#ifdef _WIN32_WCE
   char tmp;
   _asm
   {
      mov al, Index
      pushfd
      cli
      out CPU_INDEX, al
      in al,CPU_DATA
      or al, Data
      out CPU_DATA, al
      popfd
      mov tmp, al
   }
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
}

_inline UCHAR AND_CSC_REGISTERS(UINT8 Index, UINT8 Data)
// JDW - New function to prevent any possibility of conflicts
{
#ifdef _WIN32_WCE
   char tmp;
   _asm
   {
      mov al, Index
      pushfd
      cli
      out CPU_INDEX, al
      in al,CPU_DATA
      and al, Data
      out CPU_DATA, al
      popfd
      mov tmp, al
   }
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
}

_inline UCHAR AND_OR_CSC_REGISTERS(UINT8 Index, UINT8 AndMask, UINT8 OrMask)
// JDW - New function to prevent any possibility of conflicts
{
#ifdef _WIN32_WCE
   char tmp;
   _asm
   {
      mov al, Index
      pushfd
      cli
      out CPU_INDEX, al
      in  al,CPU_DATA
      and al, AndMask
      or  al, OrMask
      out CPU_DATA, al
      popfd
      mov tmp, al
   }
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
}

//////////////////////// NEXTGEN PORT 140 CONTROL REGISTER FUNCTIONS //////////////////////////

// KLUDGE CODE - Internal Use Only - Force PC/104 off while we do write to 140
#define DisablePC104() _asm {pushfd}; _asm {cli}; _outp(0x2140, _inp(0x2140) | 2);

// KLUDGE CODE - Internal Use Only - Force PC/104 off while we do write to 140
#define EnablePC104() _outp(0x2140, _inp(0x2140) & ~2); _asm {popfd};

_inline void WRITE_PORT140(UINT16 Data)
{
#ifdef _WIN32_WCE
   DisablePC104();
   _outpw(0x140, (USHORT) Data);
   EnablePC104();
#endif // _WIN32_WCE
};

_inline UINT16 READ_PORT140()
{
#ifdef _WIN32_WCE
   UINT16 tmp;
   DisablePC104();
   tmp = _inpw(0x140);
   EnablePC104();
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
};

_inline UINT16 OR_PORT140(UINT16 Data)
{
#ifdef _WIN32_WCE
   UINT16 tmp;
   DisablePC104();
   tmp = _inpw(0x140) | Data;
   _outpw(0x140, (USHORT) tmp);
   EnablePC104();
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
};

_inline UINT16 AND_PORT140(UINT16 Data)
{
#ifdef _WIN32_WCE
   UINT16 tmp;
   DisablePC104();
   tmp = _inpw(0x140) & Data;
   _outpw(0x140, (USHORT) tmp);
   EnablePC104();
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
};

_inline UINT16 AND_OR_PORT140(UINT16 AndMask, UINT16 OrMask)
{
#ifdef _WIN32_WCE
   UINT16 tmp;
   DisablePC104();
   tmp = _inpw(0x140) & AndMask | OrMask;
   _outpw(0x140, (USHORT) tmp);
   EnablePC104();
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
};

//////////////////////// GRAPHICS REGISTER FUNCTIONS ////////////////////////////////////
//
// Internal graphics adapter index and data ports.
//

#define IGC_INDEX 0x3d4
#define IGC_DATA  0x3d5


_inline void WRITE_IGC_PORT(UINT8 Index, UINT8 Data)
// JDW Save's flags now, uses word write
{
#ifdef _WIN32_WCE
   _asm
   {
       mov al, Index
       mov ah, Data
       mov dx, IGC_INDEX
       out dx, ax
    }
#endif // _WIN32_WCE
}


_inline UCHAR READ_IGC_PORT(UINT8 Index) {
// JDW Save's flags now
#ifdef _WIN32_WCE
   char tmp;
   _asm {
      mov al, Index
      mov dx, IGC_INDEX
      pushfd
      cli
      out dx, al
      inc dl
      in  al, dx
      popfd
      mov tmp, al
   }
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
}

_inline UCHAR OR_IGC_PORT(UINT8 Index, UINT8 Data)
// JDW - New function to prevent any possibility of conflicts
{
#ifdef _WIN32_WCE
   char tmp;
   _asm
   {
      mov al, Index
      mov dx, IGC_INDEX
      pushfd
      cli
      out dx, al
      inc dl
      in  al, dx
      or  al, Data
      out dx, al
      popfd
      mov tmp, al
   }
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
}

_inline UCHAR AND_IGC_PORT(UINT8 Index, UINT8 Data)
// JDW - New function to prevent any possibility of conflicts
{
#ifdef _WIN32_WCE
   char tmp;
   _asm
   {
      mov al, Index
      mov dx, IGC_INDEX
      pushfd
      cli
      out dx, al
      inc dl
      in  al, dx
      and al, Data
      out dx, al
      popfd
      mov tmp, al
   }
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
}

_inline UCHAR AND_OR_IGC_PORT(UINT8 Index, UINT8 AndMask, UINT8 OrMask)
// JDW - New function to prevent any possibility of conflicts
{
#ifdef _WIN32_WCE
   char tmp;
   _asm
   {
      mov al, Index
      mov dx, IGC_INDEX
      pushfd
      cli
      out dx, al
      inc dl
      in  al, dx
      and al, AndMask
      or  al, OrMask
      out dx, al
      popfd
      mov tmp, al
   }
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
}

//////////////////////// PCMCIA REGISTER FUNCTIONS ////////////////////////////////////
//
// Internal graphics adapter index and data ports.
//

#define PCMCIA_INDEX_PORT  0x3e0
#define PCMCIA_DATA_PORT   0x3e1
#define SOCKET_B_OFFSET    0x40     // Socket B Offset...


_inline void LoadIndexAL(UINT8 Index, UINT32 Socket)
// Computer whether to use the Index to Socket A or B, put index in to AL
{
   _asm
   {
      mov al, Index
      mov edi, Socket
      or  edi, edi
      jz  SocketA
      or  al, SOCKET_B_OFFSET
   SocketA:
   }
}

_inline UCHAR READ_82365SL(UINT8 Index, UINT32 Socket)
// JDW Save's flags now
{
#ifdef _WIN32_WCE
   char tmp;
   LoadIndexAL(Index, Socket);
   _asm
   {
      mov dx, PCMCIA_INDEX_PORT
      pushfd
      cli
      out dx, al
      inc edx
      in  al, dx
      popfd
      mov tmp, al
   }
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
}

//static int BLEH = 0;
_inline void WRITE_82365SL(UINT8 Index, UINT8 Data, UINT32 Socket)
// JDW Save's flags now, uses word write
{
#ifdef _WIN32_WCE
#ifdef DEBUG
   NKDbgPrintfW(TEXT("WRITE %x:%x=%x\r\n"), Socket, Index, Data);
#endif
   LoadIndexAL(Index, Socket);
   _asm
   {
       mov ah, Data
       mov dx, PCMCIA_INDEX_PORT
       out dx, ax
    }
#endif // _WIN32_WCE
}


_inline UCHAR OR_82365SL(UINT8 Index, UINT8 Data, UINT32 Socket)
// JDW - New function to prevent any possibility of conflicts
{
#ifdef _WIN32_WCE
   char tmp;
#ifdef DEBUG
   NKDbgPrintfW(TEXT("OR %x:%x=%x\r\n"), Socket, Index, Data);
#endif
   LoadIndexAL(Index, Socket);
   _asm
   {
      mov dx, PCMCIA_INDEX_PORT
      pushfd
      cli
      out dx, al
      inc edx
      in  al, dx
      or  al, Data
      out dx, al
      popfd
      mov tmp, al
   }
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
}

_inline UCHAR AND_82365SL(UINT8 Index, UINT8 Data, UINT32 Socket)
// JDW - New function to prevent any possibility of conflicts
{
#ifdef _WIN32_WCE
   char tmp;
#ifdef DEBUG
   NKDbgPrintfW(TEXT("AND %x:%x=%x\r\n"), Socket, Index, Data);
#endif
   LoadIndexAL(Index, Socket);
   _asm
   {
      mov dx, PCMCIA_INDEX_PORT
      pushfd
      cli
      out dx, al
      inc edx
      in  al, dx
      and al, Data
      out dx, al
      popfd
      mov tmp, al
   }
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
}

_inline UCHAR AND_OR_82365SL(UINT8 Index, UINT8 AndMask, UINT8 OrMask, UINT32 Socket)
// JDW - New function to prevent any possibility of conflicts
{
#ifdef _WIN32_WCE
   char tmp;
#ifdef DEBUG
   NKDbgPrintfW(TEXT("ANDOR %x:%x=&%x |%x\r\n"), Socket, Index, AndMask, OrMask);
#endif
   LoadIndexAL(Index, Socket);
   _asm
   {
      mov dx, PCMCIA_INDEX_PORT
      pushfd
      cli
      out dx, al
      inc edx
      in  al, dx
      and al, AndMask
      or  al, OrMask
      out dx, al
      popfd
      mov tmp, al
   }
   return tmp;
#else
   return 0;
#endif // _WIN32_WCE
}


#ifdef _WIN32_WCE
//
// Function protoypes
//

VOID
PICEnableInterrupt(
             UCHAR ucInterrupt,
             BOOL bEnabled
             );
#endif // _WIN32_WCE

#endif // _PROTEUS_H_
